package com.agendaonline.dao;

import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.List;

import javax.persistence.EntityManager;
import javax.persistence.EntityManagerFactory;
import javax.persistence.Persistence;

import org.jdom2.Attribute;
import org.jdom2.Document;
import org.jdom2.Element;
import org.jdom2.JDOMException;
import org.jdom2.input.SAXBuilder;
import org.jdom2.output.Format;
import org.jdom2.output.XMLOutputter;

import com.agendaonline.model.Paciente;
import com.agendaonline.dao.Conexao;

public class PacienteDAOJDBC {
	private Connection conexao;
	
	private static EntityManagerFactory factory;
	
	static{
		//cria as tabelas
		factory = Persistence.createEntityManagerFactory("Agenda");
	}
	//persiste os dados.
	public static EntityManager getEntityManager(){
		
		return factory.createEntityManager();
	}
	
	public static void close(){
		factory.close();
	}
	
	public void cadastrar(Paciente pac) throws SQLException {

		try {
			Connection conexao = Conexao.Conectar();
			PreparedStatement stmt = conexao.prepareStatement("INSERT INTO tblpaciente (idpaciente, nome, data, cpf, email) VALUES (?,?,?,?,?)");
			stmt.setInt(1, pac.getId());
			stmt.setString(2, pac.getNome().toString());
			stmt.setString(3, pac.getData().toString());
			stmt.setString(4, pac.getCpf().toString());
			stmt.setString(5, pac.getEmail().toString());
			stmt.execute();
			stmt.close();
		} catch (SQLException e) {
			System.out.println(e.getMessage());
		} finally {
			Conexao.Fechar();
		}
	}
	
	public void removerPorId(int id) throws SQLException{
		 
		try {
			 Connection conexao = Conexao.Conectar();
			 PreparedStatement stmt = conexao.prepareStatement("DELETE FROM tblpaciente WHERE idpaciente = ?"); 		 
			 stmt.setInt(1, id); 
			 stmt.execute();
			 stmt.close();	 
		 } catch (SQLException e) {
			 System.out.println(e.getMessage());
		 }finally{
			 Conexao.Fechar();
		 }
	}
		 
	public void atualizar(Paciente pac, int id) throws SQLException{		 
		 
		 try {
			 Connection conexao = Conexao.Conectar();
			 PreparedStatement stmt = conexao.prepareStatement("UPDATE tblpaciente SET nome = ?, data = ?, cpf = ?, email = ? WHERE idpaciente = ?"); 		 
		 
			 stmt.setString(1, pac.getNome());
			 stmt.setString(2, pac.getData());
			 stmt.setString(3, pac.getCpf());
			 stmt.setString(4, pac.getEmail());
			 stmt.setInt(5, id);
			 stmt.execute();
			 stmt.close();		 
		 } catch (SQLException e) {
			 System.out.println(e.getMessage());
		 }finally{
			 Conexao.Fechar();
		 }
	}
	
	public Paciente lerPorId(int id) throws SQLException{		 
		 Paciente p = new Paciente();
		 try {
			 Connection conexao = Conexao.Conectar();
			 PreparedStatement stmt = conexao.prepareStatement("SELECT * FROM tblpaciente WHERE idpaciente = ?"); 		  
			 stmt.setInt(1, id);
			 ResultSet rs = stmt.executeQuery();
			 
			 while (rs.next()) {
				 p.setNome(rs.getString("nome"));
				 p.setData(rs.getString("data"));
				 p.setCpf(rs.getString("cpf"));
				 p.setEmail(rs.getString("email"));				 
			}	
		 } catch (SQLException e) {
			 System.out.println(e.getMessage());
		 }finally{
			 Conexao.Fechar();
		 }
		return p;
	}
	
	public void importarXML() throws Exception {
		SAXBuilder builder = new SAXBuilder();
		File arquivoXML = new File("arquivo.xml");
		Paciente pac = new Paciente();
		
		try{
			Document doc = (Document) builder.build(arquivoXML);
			Element root = doc.getRootElement();	
			List<Element> lista = root.getChildren("paciente");
			
			for(int i =0;i < lista.size(); i++)
			{
				Element paciente = lista.get(i);
				
				pac.setId(Integer.parseInt(paciente.getAttributeValue("idpaciente")));
				pac.setNome(paciente.getChildText("nome"));
				pac.setData(paciente.getChildText("data"));
				pac.setCpf(paciente.getChildText("cpf"));
				pac.setEmail(paciente.getChildText("email"));
				cadastrar(pac);
			}
		}catch(JDOMException e)
		{
			e.getMessage();
		}catch(IOException e)
		{
			e.getMessage();
		}
	}
	
	public void exportarXML() throws Exception
	{
		Element agenda = new Element("agenda");
		Document doc = new Document(agenda);
		XMLOutputter xmlOutput = new XMLOutputter();
		
		try {
			Connection conexao = Conexao.Conectar();
			PreparedStatement stmt = conexao.prepareStatement("SELECT * FROM tblpaciente");
			ResultSet rs = stmt.executeQuery();

			while (rs.next()) {
				Paciente pac = new Paciente();
				
				pac.setId(rs.getInt("idpaciente"));
				pac.setNome(rs.getString("nome"));
				pac.setData(rs.getString("data"));
				pac.setCpf(rs.getString("cpf"));
				pac.setEmail(rs.getString("email"));
				
				Element paciente = new Element("paciente");
				
				paciente.setAttribute(new Attribute("idpaciente", Integer.toString(pac.getId())));
				paciente.addContent(new Element("nome").setText(pac.getNome()));
				paciente.addContent(new Element("data").setText(pac.getData()));
				paciente.addContent(new Element("cpf").setText(pac.getCpf()));
				paciente.addContent(new Element("email").setText(pac.getEmail()));


				doc.getRootElement().addContent(paciente);

			}
			
			xmlOutput.setFormat(Format.getPrettyFormat());

			try {
				xmlOutput.output(doc, new FileWriter("arquivo.xml"));
				System.out.println("Arquivo gerado com sucesso!");

			} 
			catch (IOException e) {

				System.err.println(e.getMessage());
			}
		
			
			rs.close();
			stmt.close();
			
		} catch (SQLException e) {
			System.out.println(e.getMessage());
		}finally {
			Conexao.Fechar();
		}
		
	}

	public List<Paciente> lerTodos() throws SQLException {
		

		List<Paciente> pacientes = new ArrayList<Paciente>();
		
		try {
			Connection conexao = Conexao.Conectar();
			PreparedStatement stmt = conexao.prepareStatement("SELECT * FROM tblpaciente");
			ResultSet rs = stmt.executeQuery();

			while (rs.next()) {
				Paciente Pa = new Paciente();
				
				Pa.setId(rs.getInt("idpaciente"));
				Pa.setNome(rs.getString("nome"));
				Pa.setData(rs.getString("data"));
				Pa.setCpf(rs.getString("cpf"));
				Pa.setEmail(rs.getString("email"));
				
				pacientes.add(Pa);

			}
			
			rs.close();
			stmt.close();
			
		} catch (SQLException e) {
			System.out.println(e.getMessage());
		}finally {
			Conexao.Fechar();
		}
		return pacientes;
	}
	
	
	
	
	
	
	
	
	
	
	
	
	
	
}
